#!/usr/bin/perl
#
# ICMP-ping script to be used with netdog.pl
#
# (c) Magnus Bodin 2000-2001; Ordinary BSD License applies.
#
#

use strict;
use Getopt::Std;
use IPC::Open3;

my %opt;
getopt('tc', \%opt);

sub icmp_ping_host ($$$)
{
    my ($count, $timeout, $host) = @_;
	my($wtr, $rdr, $err);
    my @result;

    # SECURITY MEASURES. We don't want to call the shell with bad stuff
    $count = scalar(0 + $count) || 1;
    $timeout = scalar(0 + $timeout) || 10;
    $host = lc $host;
    $host =~ s/[^-\:\.a-z0-9]//g;

    my $pid = open3($wtr, $rdr, $err,
	'ping', ('-c', ${count}, '-w', ${timeout}, '-i', '0.3', $host));
    close $wtr;
    my $ping = join '',<$rdr>;
    my $err = join '',<$err>;

    my ($tx, $rx, $min, $avg, $max, $mdev);
    if ($ping =~ m#^(\d+).+transmitted\D+(\d+)\D+received#m)
    {
        ($tx, $rx) = ($1, $2);
        if ($ping =~ m#^min/avg/max/mdev\D+([\d\.]+)/([\d\.]+)/([\d\.]+)/([\d\.]+)\s#m)
        {
            ($min, $avg, $max, $mdev) = ($1, $2, $3, $4);
        }
    }

    my $pl = 100;
    $pl = 100 * (($tx - $rx) / $tx) if ($tx > 0) && ($rx > 0);
    return [$pl, $tx, $rx, $min, $avg, $max, $mdev, $ping];
}

$opt{h} = shift @ARGV;

unless ($opt{h})
{
    print <<"--EOD";
$0
netdog compatible icmp-ping
Usage: $0 [-c <packetcount>] [-t <timeout in seconds>] <host>
--EOD
    exit -2;
}

$opt{c} ||= 5;
$opt{t} ||= 6;

$SIG{ALRM} = sub { print "-1\n"; exit };

alarm($opt{t} + 2);

my $ping = icmp_ping_host($opt{c}, $opt{t}, $opt{h});

print join "\n", @{$ping};

