#!/usr/bin/perl -w
#
# tcp_lineproto-ping script to be used with netdog.pl
# 
# (c) Magnus Bodin 2000-2003; Ordinary BSD License applies.
#

use strict;
use IO::Socket;
use Getopt::Std;

my %opt;
getopt('t', \%opt);

if(@ARGV < 1) { print "Usage: $0 <URL> [-t <timeout>]\n"; exit; }

my ($URL) = @ARGV;

sub parse_url ($)
{
    my ($URL) = @_;
    
    my $service = 'http';
    my $host = lc $URL;
    my $port = 0;
    my $doc = '';
    
    if ($URL =~ m#^(\w+)://([^/]+)(/.*)?$#)
    {
        $service = $1;
        $host = $2;
        $doc ||= $3;
    }

    $doc ||= '/';
    
    if ($host =~ /^(.+)\:(\d+)$/)
    {
        ($host, $port) = ($1, $2);
    }
    
    unless ($port)
    {
        my @serv = getservbyname($service, 'tcp');
        $port = $serv[2] if @serv;
    }
    
    my $username = '';
    my $password = '';
    if ($host =~ /^([^\@]+)\@(.+)$/)
    {
        $username = $1;
        $host = $2;
        
        if ($username =~ /^(.+)\:(.+)$/)
        {
            $username = $1;
            $password = $2;
        }
    }

    return [ lc $service, $port, lc $host, $username, $password, $doc ];
}

sub tcp_connect
{
    my ($host, $port);
    my $connection = 1;
    my $result = '';
    my $socket;
    $socket = IO::Socket::INET->new(
        Proto => 'tcp',
        PeerAddr => $_[0], 
        PeerPort => $_[1],
    ) or ($connection, $socket, $result) = (0, 0, $!);
    return [ $connection, $socket, $result ];
}


my $u = parse_url($URL);
my $socket;
my $remote;
$opt{t} ||= 0;

if ($opt{t} > 0)
{
    $SIG{ALRM} = sub { print "-2\n"; $socket->close; exit -2; };
    alarm($opt{t});
}

my %prot_map = ( 'tcp' => 1, 'http' => 2, 'smtp' => 3 );
my $mapped = $prot_map{$$u[0]};

if ($mapped)
{
    my $s = tcp_connect($$u[2], $$u[1]);
    $socket = $$s[1];
    if ($$s[0])
    {
	# TCP
	if ($mapped == 1)
	{
             print "0\n";
	}
	# HTTP
        elsif ($mapped == 2)
        {
             print $socket "GET / HTTP/1.0\n\n";
             $remote = join '', <$socket>;
             print "0\n";
        }
	# SMTP
        elsif ($mapped == 3)
        {
             print $socket "HELO $$u[3]\n";
             $remote = <$socket>;
             print $socket "QUIT\n";
             $socket->close;
             print "0\n";
        }
        $socket->close;
    }
    else
    {
        print "-1\n";
    }
}
else
{
    print "-3\n";
}

